@@ -1,4 +1,4 @@
-// Copyright 2016-2020, Pulumi Corporation.
+// Copyright 2016-2021, Pulumi Corporation.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -35,6 +35,8 @@ import (
 
 	"github.com/pulumi/pulumi/pkg/v3/codegen"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/diag"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/util/cmdutil"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
 )
 
@@ -93,13 +95,15 @@ type pkgContext struct {
 	resources       []*schema.Resource
 	functions       []*schema.Function
 	// schemaNames tracks the names of types/resources as specified in the schema
-	schemaNames   codegen.StringSet
-	names         codegen.StringSet
-	renamed       map[string]string
-	functionNames map[*schema.Function]string
-	needsUtils    bool
-	tool          string
-	packages      map[string]*pkgContext
+	schemaNames codegen.StringSet
+	names       codegen.StringSet
+	renamed     map[string]string
+	// duplicateTokens tracks tokens that exist for both types and resources
+	duplicateTokens map[string]bool
+	functionNames   map[*schema.Function]string
+	needsUtils      bool
+	tool            string
+	packages        map[string]*pkgContext
 
 	// Name overrides set in GoPackageInfo
 	modToPkg         map[string]string // Module name -> package name
@@ -145,13 +149,9 @@ func (pkg *pkgContext) tokenToType(tok string) string {
 		newName, renamed := modPkg.renamed[name]
 		if renamed {
 			name = newName
-		} else if modPkg.names.Has(name) {
-			// If the package containing the type's token already has a resource with the
-			// same name, add a `Type` suffix.
-			newName = name + "Type"
-			modPkg.renamed[name] = newName
-			modPkg.names.Add(newName)
-			name = newName
+		} else if modPkg.duplicateTokens[tok] {
+			// maintain support for duplicate tokens for types and resources in Kubernetes
+			name = name + "Type"
 		}
 	}
 
@@ -211,7 +211,17 @@ func tokenToName(tok string) string {
 	return Title(components[2])
 }
 
-func resourceName(r *schema.Resource) string {
+// disambiguatedResourceName gets the name of a resource as it should appear in source, resolving conflicts in the process.
+func disambiguatedResourceName(r *schema.Resource, pkg *pkgContext) string {
+	name := rawResourceName(r)
+	if renamed, ok := pkg.renamed[name]; ok {
+		name = renamed
+	}
+	return name
+}
+
+// rawResourceName produces raw resource name translated from schema type token without resolving conflicts or dupes.
+func rawResourceName(r *schema.Resource) string {
 	if r.IsProvider {
 		return "Provider"
 	}
@@ -706,7 +716,32 @@ func (pkg *pkgContext) getInputUsage(name string) string {
 	}, "\n")
 }
 
+type genInputImplementationArgs struct {
+	name            string
+	receiverType    string
+	elementType     string
+	ptrMethods      bool
+	toOutputMethods bool
+	resourceType    bool
+}
+
 func genInputImplementation(w io.Writer, name, receiverType, elementType string, ptrMethods, resourceType bool) {
+	genInputImplementationWithArgs(w, genInputImplementationArgs{
+		name:            name,
+		receiverType:    receiverType,
+		elementType:     elementType,
+		ptrMethods:      ptrMethods,
+		resourceType:    resourceType,
+		toOutputMethods: true,
+	})
+}
+
+func genInputImplementationWithArgs(w io.Writer, genArgs genInputImplementationArgs) {
+	name := genArgs.name
+	receiverType := genArgs.receiverType
+	elementType := genArgs.elementType
+	resourceType := genArgs.resourceType
+
 	fmt.Fprintf(w, "func (%s) ElementType() reflect.Type {\n", receiverType)
 	if resourceType {
 		fmt.Fprintf(w, "\treturn reflect.TypeOf((*%s)(nil))\n", elementType)
@@ -715,15 +750,17 @@ func genInputImplementation(w io.Writer, name, receiverType, elementType string,
 	}
 	fmt.Fprintf(w, "}\n\n")
 
-	fmt.Fprintf(w, "func (i %s) To%sOutput() %sOutput {\n", receiverType, Title(name), name)
-	fmt.Fprintf(w, "\treturn i.To%sOutputWithContext(context.Background())\n", Title(name))
-	fmt.Fprintf(w, "}\n\n")
+	if genArgs.toOutputMethods {
+		fmt.Fprintf(w, "func (i %s) To%sOutput() %sOutput {\n", receiverType, Title(name), name)
+		fmt.Fprintf(w, "\treturn i.To%sOutputWithContext(context.Background())\n", Title(name))
+		fmt.Fprintf(w, "}\n\n")
 
-	fmt.Fprintf(w, "func (i %s) To%sOutputWithContext(ctx context.Context) %sOutput {\n", receiverType, Title(name), name)
-	fmt.Fprintf(w, "\treturn pulumi.ToOutputWithContext(ctx, i).(%sOutput)\n", name)
-	fmt.Fprintf(w, "}\n\n")
+		fmt.Fprintf(w, "func (i %s) To%sOutputWithContext(ctx context.Context) %sOutput {\n", receiverType, Title(name), name)
+		fmt.Fprintf(w, "\treturn pulumi.ToOutputWithContext(ctx, i).(%sOutput)\n", name)
+		fmt.Fprintf(w, "}\n\n")
+	}
 
-	if ptrMethods {
+	if genArgs.ptrMethods {
 		fmt.Fprintf(w, "func (i %s) To%sPtrOutput() %sPtrOutput {\n", receiverType, Title(name), name)
 		fmt.Fprintf(w, "\treturn i.To%sPtrOutputWithContext(context.Background())\n", Title(name))
 		fmt.Fprintf(w, "}\n\n")
@@ -1054,13 +1091,7 @@ func (pkg *pkgContext) genInputTypes(w io.Writer, t *schema.ObjectType, details
 	// Generate the plain inputs.
 	pkg.genInputInterface(w, name)
 
-	printComment(w, t.Comment, false)
-	fmt.Fprintf(w, "type %sArgs struct {\n", name)
-	for _, p := range t.Properties {
-		printCommentWithDeprecationMessage(w, p.Comment, p.DeprecationMessage, true)
-		fmt.Fprintf(w, "\t%s %s `pulumi:\"%s\"`\n", Title(p.Name), pkg.typeString(p.Type), p.Name)
-	}
-	fmt.Fprintf(w, "}\n\n")
+	pkg.genInputArgsStruct(w, name+"Args", t)
 
 	genInputImplementation(w, name, name+"Args", name, details.ptrElement, false)
 
@@ -1098,14 +1129,43 @@ func (pkg *pkgContext) genInputTypes(w io.Writer, t *schema.ObjectType, details
 	}
 }
 
-func (pkg *pkgContext) genOutputTypes(w io.Writer, t *schema.ObjectType, details *typeDetails) {
-	contract.Assert(!t.IsInputShape())
-
-	name := pkg.tokenToType(t.Token)
+func (pkg *pkgContext) genInputArgsStruct(w io.Writer, typeName string, t *schema.ObjectType) {
+	contract.Assert(t.IsInputShape())
 
 	printComment(w, t.Comment, false)
+	fmt.Fprintf(w, "type %s struct {\n", typeName)
+	for _, p := range t.Properties {
+		printCommentWithDeprecationMessage(w, p.Comment, p.DeprecationMessage, true)
+		fmt.Fprintf(w, "\t%s %s `pulumi:\"%s\"`\n", Title(p.Name), pkg.typeString(p.Type), p.Name)
+	}
+	fmt.Fprintf(w, "}\n\n")
+}
 
-	genOutputType(w, name, name, details.ptrElement, false)
+type genOutputTypesArgs struct {
+	t *schema.ObjectType
+
+	// optional type name override
+	name string
+}
+
+func (pkg *pkgContext) genOutputTypes(w io.Writer, genArgs genOutputTypesArgs) {
+	t := genArgs.t
+	details := pkg.detailsForType(t)
+
+	contract.Assert(!t.IsInputShape())
+
+	name := genArgs.name
+	if name == "" {
+		name = pkg.tokenToType(t.Token)
+	}
+
+	printComment(w, t.Comment, false)
+	genOutputType(w,
+		name,               /* baseName */
+		name,               /* elementType */
+		details.ptrElement, /* ptrMethods */
+		false,              /* resourceType */
+	)
 
 	for _, p := range t.Properties {
 		printCommentWithDeprecationMessage(w, p.Comment, p.DeprecationMessage, false)
@@ -1117,7 +1177,8 @@ func (pkg *pkgContext) genOutputTypes(w io.Writer, t *schema.ObjectType, details
 			propName = "Get" + propName
 		}
 		fmt.Fprintf(w, "func (o %sOutput) %s() %s {\n", name, propName, outputType)
-		fmt.Fprintf(w, "\treturn o.ApplyT(func (v %s) %s { return v.%s }).(%s)\n", name, applyType, Title(p.Name), outputType)
+		fmt.Fprintf(w, "\treturn o.ApplyT(func (v %s) %s { return v.%s }).(%s)\n",
+			name, applyType, Title(p.Name), outputType)
 		fmt.Fprintf(w, "}\n\n")
 	}
 
@@ -1243,7 +1304,7 @@ func (pkg *pkgContext) getDefaultValue(dv *schema.DefaultValue, t schema.Type) (
 }
 
 func (pkg *pkgContext) genResource(w io.Writer, r *schema.Resource, generateResourceContainerTypes bool) error {
-	name := resourceName(r)
+	name := disambiguatedResourceName(r, pkg)
 
 	printCommentWithDeprecationMessage(w, r.Comment, r.DeprecationMessage, false)
 	fmt.Fprintf(w, "type %s struct {\n", name)
@@ -1258,6 +1319,7 @@ func (pkg *pkgContext) genResource(w io.Writer, r *schema.Resource, generateReso
 	}
 
 	var secretProps []*schema.Property
+
 	for _, p := range r.Properties {
 		printCommentWithDeprecationMessage(w, p.Comment, p.DeprecationMessage, true)
 		fmt.Fprintf(w, "\t%s %s `pulumi:\"%s\"`\n", Title(p.Name), pkg.outputType(p.Type), p.Name)
@@ -1358,6 +1420,8 @@ func (pkg *pkgContext) genResource(w io.Writer, r *schema.Resource, generateReso
 		fmt.Fprintf(w, "\t})\n")
 		fmt.Fprintf(w, "\topts = append(opts, aliases)\n")
 	}
+
+	// Setup secrets
 	if len(secretProps) > 0 {
 		for _, p := range secretProps {
 			fmt.Fprintf(w, "\tif args.%s != nil {\n", Title(p.Name))
@@ -1372,6 +1436,22 @@ func (pkg *pkgContext) genResource(w io.Writer, r *schema.Resource, generateReso
 		fmt.Fprintf(w, "\topts = append(opts, secrets)\n")
 	}
 
+	// Setup replaceOnChange
+	replaceOnChangesProps, errList := r.ReplaceOnChanges()
+	for _, err := range errList {
+		cmdutil.Diag().Warningf(&diag.Diag{Message: err.Error()})
+	}
+	replaceOnChangesStrings := schema.PropertyListJoinToString(replaceOnChangesProps,
+		func(x string) string { return x })
+	if len(replaceOnChangesProps) > 0 {
+		fmt.Fprint(w, "\treplaceOnChanges := pulumi.ReplaceOnChanges([]string{\n")
+		for _, p := range replaceOnChangesStrings {
+			fmt.Fprintf(w, "\t\t%q,\n", p)
+		}
+		fmt.Fprint(w, "\t})\n")
+		fmt.Fprint(w, "\topts = append(opts, replaceOnChanges)\n")
+	}
+
 	// Finally make the call to registration.
 	fmt.Fprintf(w, "\tvar resource %s\n", name)
 	if r.IsComponent {
@@ -1618,10 +1698,24 @@ func (pkg *pkgContext) genResource(w io.Writer, r *schema.Resource, generateReso
 	return nil
 }
 
-func (pkg *pkgContext) genFunction(w io.Writer, f *schema.Function) {
-	// If the function starts with New or Get, it will conflict; so rename them.
-	name := pkg.functionNames[f]
+func (pkg *pkgContext) genFunctionCodeFile(f *schema.Function) string {
+	importsAndAliases := map[string]string{}
+	pkg.getImports(f, importsAndAliases)
+	buffer := &bytes.Buffer{}
 
+	var imports []string
+	if f.NeedsOutputVersion() {
+		imports = []string{"context", "reflect"}
+	}
+
+	pkg.genHeader(buffer, imports, importsAndAliases)
+	pkg.genFunction(buffer, f)
+	pkg.genFunctionOutputVersion(buffer, f)
+	return buffer.String()
+}
+
+func (pkg *pkgContext) genFunction(w io.Writer, f *schema.Function) {
+	name := pkg.functionName(f)
 	printCommentWithDeprecationMessage(w, f.Comment, f.DeprecationMessage, false)
 
 	// Now, emit the function signature.
@@ -1671,20 +1765,91 @@ func (pkg *pkgContext) genFunction(w io.Writer, f *schema.Function) {
 	// If there are argument and/or return types, emit them.
 	if f.Inputs != nil {
 		fmt.Fprintf(w, "\n")
-		pkg.genPlainType(w, fmt.Sprintf("%sArgs", name), f.Inputs.Comment, "", f.Inputs.Properties)
+		pkg.genPlainType(w, pkg.functionArgsTypeName(f), f.Inputs.Comment, "", f.Inputs.Properties)
 	}
 	if f.Outputs != nil {
 		fmt.Fprintf(w, "\n")
-		pkg.genPlainType(w, fmt.Sprintf("%sResult", name), f.Outputs.Comment, "", f.Outputs.Properties)
+		pkg.genPlainType(w, pkg.functionResultTypeName(f), f.Outputs.Comment, "", f.Outputs.Properties)
 	}
 }
 
+func (pkg *pkgContext) functionName(f *schema.Function) string {
+	// If the function starts with New or Get, it will conflict; so rename them.
+	name, hasName := pkg.functionNames[f]
+
+	if !hasName {
+		panic(fmt.Sprintf("No function name found for %v", f))
+	}
+
+	return name
+}
+
+func (pkg *pkgContext) functionArgsTypeName(f *schema.Function) string {
+	name := pkg.functionName(f)
+	return fmt.Sprintf("%sArgs", name)
+}
+
+func (pkg *pkgContext) functionResultTypeName(f *schema.Function) string {
+	name := pkg.functionName(f)
+	return fmt.Sprintf("%sResult", name)
+}
+
+func (pkg *pkgContext) genFunctionOutputVersion(w io.Writer, f *schema.Function) {
+	if !f.NeedsOutputVersion() {
+		return
+	}
+
+	originalName := pkg.functionName(f)
+	name := originalName + "Output"
+	originalResultTypeName := pkg.functionResultTypeName(f)
+	resultTypeName := originalResultTypeName + "Output"
+
+	code := `
+func ${fn}Output(ctx *pulumi.Context, args ${fn}OutputArgs, opts ...pulumi.InvokeOption) ${outputType} {
+	return pulumi.ToOutputWithContext(context.Background(), args).
+		ApplyT(func(v interface{}) (${fn}Result, error) {
+			args := v.(${fn}Args)
+			r, err := ${fn}(ctx, &args, opts...)
+			return *r, err
+		}).(${outputType})
+}
+
+`
+	code = strings.ReplaceAll(code, "${fn}", originalName)
+	code = strings.ReplaceAll(code, "${outputType}", resultTypeName)
+	fmt.Fprintf(w, code)
+
+	pkg.genInputArgsStruct(w, name+"Args", f.Inputs.InputShape)
+
+	genInputImplementationWithArgs(w, genInputImplementationArgs{
+		name:         name + "Args",
+		receiverType: name + "Args",
+		elementType:  pkg.functionArgsTypeName(f),
+	})
+
+	pkg.genOutputTypes(w, genOutputTypesArgs{
+		t:    f.Outputs,
+		name: originalResultTypeName,
+	})
+
+	// Assuming the file represented by `w` only has one function,
+	// generate an `init()` for Output type init.
+	initCode := `
+func init() {
+        pulumi.RegisterOutputType(${outputType}{})
+}
+
+`
+	initCode = strings.ReplaceAll(initCode, "${outputType}", resultTypeName)
+	fmt.Fprintf(w, initCode)
+}
+
 func (pkg *pkgContext) genType(w io.Writer, obj *schema.ObjectType) {
 	contract.Assert(!obj.IsInputShape())
 
 	pkg.genPlainType(w, pkg.tokenToType(obj.Token), obj.Comment, "", obj.Properties)
 	pkg.genInputTypes(w, obj.InputShape, pkg.detailsForType(obj))
-	pkg.genOutputTypes(w, obj, pkg.detailsForType(obj))
+	pkg.genOutputTypes(w, genOutputTypesArgs{t: obj})
 }
 
 func (pkg *pkgContext) addSuffixesToName(typ schema.Type, name string) []string {
@@ -2153,7 +2318,7 @@ func (pkg *pkgContext) genResourceModule(w io.Writer) {
 
 			registrations.Add(tokenToModule(r.Token))
 			fmt.Fprintf(w, "\tcase %q:\n", r.Token)
-			fmt.Fprintf(w, "\t\tr = &%s{}\n", resourceName(r))
+			fmt.Fprintf(w, "\t\tr = &%s{}\n", disambiguatedResourceName(r, pkg))
 		}
 		fmt.Fprintf(w, "\tdefault:\n")
 		fmt.Fprintf(w, "\t\treturn nil, fmt.Errorf(\"unknown resource type: %%s\", typ)\n")
@@ -2232,6 +2397,7 @@ func generatePackageContextMap(tool string, pkg *schema.Package, goInfo GoPackag
 				names:            codegen.NewStringSet(),
 				schemaNames:      codegen.NewStringSet(),
 				renamed:          map[string]string{},
+				duplicateTokens:  map[string]bool{},
 				functionNames:    map[*schema.Function]string{},
 				tool:             tool,
 				modToPkg:         goInfo.ModuleToPackage,
@@ -2342,21 +2508,69 @@ func generatePackageContextMap(tool string, pkg *schema.Package, goInfo GoPackag
 		}
 	}
 
+	resSeen := map[string]bool{}
+	typeSeen := map[string]bool{}
+
+	// compute set of names generated by a resource
+	// handling any potential collisions via remapping along the way
 	scanResource := func(r *schema.Resource) {
+		if resSeen[r.Token] {
+			return
+		}
+		resSeen[r.Token] = true
 		pkg := getPkgFromToken(r.Token)
 		pkg.resources = append(pkg.resources, r)
 		pkg.schemaNames.Add(tokenToName(r.Token))
 
-		pkg.names.Add(resourceName(r))
-		pkg.names.Add(resourceName(r) + "Input")
-		pkg.names.Add(resourceName(r) + "Output")
-		pkg.names.Add(resourceName(r) + "Args")
-		pkg.names.Add(camel(resourceName(r)) + "Args")
-		pkg.names.Add("New" + resourceName(r))
-		if !r.IsProvider && !r.IsComponent {
-			pkg.names.Add(resourceName(r) + "State")
-			pkg.names.Add(camel(resourceName(r)) + "State")
-			pkg.names.Add("Get" + resourceName(r))
+		getNames := func(suffix string) []string {
+			names := []string{}
+			names = append(names, rawResourceName(r)+suffix)
+			names = append(names, rawResourceName(r)+suffix+"Input")
+			names = append(names, rawResourceName(r)+suffix+"Output")
+			names = append(names, rawResourceName(r)+suffix+"Args")
+			names = append(names, camel(rawResourceName(r))+suffix+"Args")
+			names = append(names, "New"+rawResourceName(r)+suffix)
+			if !r.IsProvider && !r.IsComponent {
+				names = append(names, rawResourceName(r)+suffix+"State")
+				names = append(names, camel(rawResourceName(r))+suffix+"State")
+				names = append(names, "Get"+rawResourceName(r)+suffix)
+			}
+			return names
+		}
+
+		suffixes := []string{"", "Resource", "Res"}
+		suffix := ""
+		suffixIndex := 0
+		canGenerate := false
+
+		for !canGenerate && suffixIndex <= len(suffixes) {
+			suffix = suffixes[suffixIndex]
+			candidates := getNames(suffix)
+			conflict := false
+			for _, c := range candidates {
+				if pkg.names.Has(c) {
+					conflict = true
+				}
+			}
+			if !conflict {
+				canGenerate = true
+				break
+			}
+
+			suffixIndex++
+		}
+
+		if !canGenerate {
+			panic(fmt.Sprintf("unable to generate Go SDK, schema has unresolvable overlapping resource: %s", rawResourceName(r)))
+		}
+
+		names := getNames(suffix)
+		originalNames := getNames("")
+		for i, n := range names {
+			pkg.names.Add(n)
+			if suffix != "" {
+				pkg.renamed[originalNames[i]] = names[i]
+			}
 		}
 
 		populateDetailsForPropertyTypes(seenMap, r.InputProperties, !r.IsProvider)
@@ -2364,10 +2578,10 @@ func generatePackageContextMap(tool string, pkg *schema.Package, goInfo GoPackag
 
 		for _, method := range r.Methods {
 			if method.Function.Inputs != nil {
-				pkg.names.Add(resourceName(r) + Title(method.Name) + "Args")
+				pkg.names.Add(rawResourceName(r) + Title(method.Name) + "Args")
 			}
 			if method.Function.Outputs != nil {
-				pkg.names.Add(resourceName(r) + Title(method.Name) + "Result")
+				pkg.names.Add(rawResourceName(r) + Title(method.Name) + "Result")
 			}
 		}
 	}
@@ -2377,6 +2591,86 @@ func generatePackageContextMap(tool string, pkg *schema.Package, goInfo GoPackag
 		scanResource(r)
 	}
 
+	// compute set of names generated by a type
+	// handling any potential collisions via remapping along the way
+	scanType := func(t schema.Type) {
+		switch t := t.(type) {
+		case *schema.ObjectType:
+			pkg := getPkgFromToken(t.Token)
+			// maintain support for duplicate tokens for types and resources in Kubernetes
+			if resSeen[t.Token] {
+				pkg.duplicateTokens[t.Token] = true
+				return
+			}
+			if typeSeen[t.Token] {
+				return
+			}
+			typeSeen[t.Token] = true
+
+			name := pkg.tokenToType(t.Token)
+			getNames := func(suffix string) []string {
+				names := []string{}
+				names = append(names, name+suffix)
+				names = append(names, name+suffix+"Input")
+				names = append(names, name+suffix+"Output")
+				return names
+			}
+			suffixes := []string{"", "Type", "Typ"}
+			suffix := ""
+			suffixIndex := 0
+			canGenerate := false
+
+			for !canGenerate && suffixIndex <= len(suffixes) {
+				suffix = suffixes[suffixIndex]
+				candidates := getNames(suffix)
+				conflict := false
+				for _, c := range candidates {
+					if pkg.names.Has(c) {
+						conflict = true
+					}
+				}
+				if !conflict {
+					canGenerate = true
+					break
+				}
+
+				suffixIndex++
+			}
+
+			if !canGenerate {
+				panic(fmt.Sprintf("unable to generate Go SDK, schema has unresolvable overlapping type: %s", name))
+			}
+
+			names := getNames(suffix)
+			originalNames := getNames("")
+			for i, n := range names {
+				pkg.names.Add(n)
+				if suffix != "" {
+					pkg.renamed[originalNames[i]] = names[i]
+				}
+			}
+		default:
+			return
+		}
+	}
+
+	for _, t := range pkg.Types {
+		scanType(t)
+	}
+
+	// For fnApply function versions, we need to register any
+	// input or output property type metadata, in case they have
+	// types used in array or pointer element positions.
+	for _, f := range pkg.Functions {
+		parentOptional := false
+		if f.Inputs != nil {
+			populateDetailsForPropertyTypes(seenMap, f.Inputs.Properties, parentOptional)
+		}
+		if f.Outputs != nil {
+			populateDetailsForPropertyTypes(seenMap, f.Outputs.Properties, parentOptional)
+		}
+	}
+
 	for _, f := range pkg.Functions {
 		if f.IsMethod {
 			continue
@@ -2549,20 +2843,14 @@ func GeneratePackage(tool string, pkg *schema.Package) (map[string][]byte, error
 				return nil, err
 			}
 
-			setFile(path.Join(mod, camel(resourceName(r))+".go"), buffer.String())
+			setFile(path.Join(mod, camel(rawResourceName(r))+".go"), buffer.String())
 		}
 
 		// Functions
 		for _, f := range pkg.functions {
-			importsAndAliases := map[string]string{}
-			pkg.getImports(f, importsAndAliases)
-
-			buffer := &bytes.Buffer{}
-			pkg.genHeader(buffer, nil, importsAndAliases)
-
-			pkg.genFunction(buffer, f)
-
-			setFile(path.Join(mod, camel(tokenToName(f.Token))+".go"), buffer.String())
+			fileName := path.Join(mod, camel(tokenToName(f.Token))+".go")
+			code := pkg.genFunctionCodeFile(f)
+			setFile(fileName, code)
 		}
 
 		knownTypes := make(map[schema.Type]struct{}, len(pkg.typeDetails))
