@@ -15,7 +15,9 @@
 package test
 
 import (
+	"bytes"
 	"encoding/json"
+	"io"
 	"io/ioutil"
 	"os"
 	"path/filepath"
@@ -116,6 +118,7 @@ func ValidateFileEquality(t *testing.T, actual, expected map[string][]byte) bool
 	ok := true
 	for name, file := range expected {
 		if !assert.Contains(t, actual, name) || !assert.Equal(t, string(file), string(actual[name]), name) {
+			t.Logf("%s did not agree", name)
 			ok = false
 		}
 	}
@@ -185,3 +188,47 @@ func CheckAllFilesGenerated(t *testing.T, actual, expected map[string][]byte) {
 		assert.Fail(t, "No content generated for expected file %s", s)
 	}
 }
+
+// Validates a transformer on a single file.
+func ValidateFileTransformer(
+	t *testing.T,
+	inputFile string,
+	expectedOutputFile string,
+	transformer func(reader io.Reader, writer io.Writer) error) {
+
+	reader, err := os.Open(inputFile)
+	if err != nil {
+		t.Error(err)
+		return
+	}
+
+	var buf bytes.Buffer
+
+	err = transformer(reader, &buf)
+	if err != nil {
+		t.Error(err)
+		return
+	}
+
+	actualBytes := buf.Bytes()
+
+	if os.Getenv("PULUMI_ACCEPT") != "" {
+		err := ioutil.WriteFile(expectedOutputFile, actualBytes, 0600)
+		if err != nil {
+			t.Error(err)
+			return
+		}
+	}
+
+	actual := map[string][]byte{expectedOutputFile: actualBytes}
+
+	expectedBytes, err := ioutil.ReadFile(expectedOutputFile)
+	if err != nil {
+		t.Error(err)
+		return
+	}
+
+	expected := map[string][]byte{expectedOutputFile: expectedBytes}
+
+	ValidateFileEquality(t, actual, expected)
+}
