@@ -1,4 +1,4 @@
-// Copyright 2016-2020, Pulumi Corporation.
+// Copyright 2016-2021, Pulumi Corporation.
 //
 // Licensed under the Apache License, Version 2.0 (the "License");
 // you may not use this file except in compliance with the License.
@@ -36,7 +36,9 @@ import (
 
 	"github.com/pulumi/pulumi/pkg/v3/codegen"
 	"github.com/pulumi/pulumi/pkg/v3/codegen/schema"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/diag"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/resource/plugin"
+	"github.com/pulumi/pulumi/sdk/v3/go/common/util/cmdutil"
 	"github.com/pulumi/pulumi/sdk/v3/go/common/util/contract"
 )
 
@@ -333,13 +335,17 @@ func printComment(w io.Writer, comment string, indent string) {
 	fmt.Fprintf(w, "%s\"\"\"\n", indent)
 }
 
-func (mod *modContext) genHeader(w io.Writer, needsSDK bool, imports imports) {
+func genStandardHeader(w io.Writer, tool string) {
 	// Set the encoding to UTF-8, in case the comments contain non-ASCII characters.
 	fmt.Fprintf(w, "# coding=utf-8\n")
 
 	// Emit a standard warning header ("do not edit", etc).
-	fmt.Fprintf(w, "# *** WARNING: this file was generated by %v. ***\n", mod.tool)
+	fmt.Fprintf(w, "# *** WARNING: this file was generated by %v. ***\n", tool)
 	fmt.Fprintf(w, "# *** Do not edit by hand unless you're certain you know what you are doing! ***\n\n")
+}
+
+func (mod *modContext) genHeader(w io.Writer, needsSDK bool, imports imports) {
+	genStandardHeader(w, mod.tool)
 
 	// If needed, emit the standard Pulumi SDK import statement.
 	if needsSDK {
@@ -385,6 +391,13 @@ func (fs fs) add(path string, contents []byte) {
 	fs[path] = contents
 }
 
+func genUtilitiesFile(tool string) []byte {
+	buffer := &bytes.Buffer{}
+	genStandardHeader(buffer, tool)
+	fmt.Fprintf(buffer, "%s", utilitiesFile)
+	return buffer.Bytes()
+}
+
 func (mod *modContext) gen(fs fs) error {
 	dir := path.Join(mod.pyPkgName, mod.mod)
 
@@ -410,10 +423,7 @@ func (mod *modContext) gen(fs fs) error {
 	// Utilities, config, readme
 	switch mod.mod {
 	case "":
-		buffer := &bytes.Buffer{}
-		mod.genHeader(buffer, false /*needsSDK*/, nil)
-		fmt.Fprintf(buffer, "%s", utilitiesFile)
-		fs.add(filepath.Join(dir, "_utilities.py"), buffer.Bytes())
+		fs.add(filepath.Join(dir, "_utilities.py"), genUtilitiesFile(mod.tool))
 		fs.add(filepath.Join(dir, "py.typed"), []byte{})
 
 		// Ensure that the top-level (provider) module directory contains a README.md file.
@@ -1266,6 +1276,16 @@ func (mod *modContext) genResource(res *schema.Resource) (string, error) {
 		fmt.Fprintf(w, "\n        opts = pulumi.ResourceOptions.merge(opts, secret_opts)\n")
 	}
 
+	replaceOnChangesProps, errList := res.ReplaceOnChanges()
+	for _, err := range errList {
+		cmdutil.Diag().Warningf(&diag.Diag{Message: err.Error()})
+	}
+	if len(replaceOnChangesProps) > 0 {
+		replaceOnChangesStrings := schema.PropertyListJoinToString(replaceOnChangesProps, PyName)
+		fmt.Fprintf(w, `        replace_on_changes = pulumi.ResourceOptions(replace_on_changes=["%s"])`, strings.Join(replaceOnChangesStrings, `", "`))
+		fmt.Fprintf(w, "\n        opts = pulumi.ResourceOptions.merge(opts, replace_on_changes)\n")
+	}
+
 	// Finally, chain to the base constructor, which will actually register the resource.
 	tok := res.Token
 	if res.IsProvider {
@@ -1572,6 +1592,9 @@ func (mod *modContext) genFunction(fun *schema.Function) (string, error) {
 		fmt.Fprintf(w, "    '%s',\n", awaitableName)
 	}
 	fmt.Fprintf(w, "    '%s',\n", name)
+	if fun.NeedsOutputVersion() {
+		fmt.Fprintf(w, "    '%s_output',\n", name)
+	}
 	fmt.Fprintf(w, "]\n\n")
 
 	if fun.DeprecationMessage != "" {
@@ -1592,47 +1615,9 @@ func (mod *modContext) genFunction(fun *schema.Function) (string, error) {
 		args = fun.Inputs.Properties
 	}
 
-	// Write out the function signature.
-	def := fmt.Sprintf("def %s(", name)
-	var indent string
-	if len(args) > 0 {
-		indent = strings.Repeat(" ", len(def))
-	}
-	fmt.Fprintf(w, def)
-	for i, arg := range args {
-		var ind string
-		if i != 0 {
-			ind = indent
-		}
-		pname := PyName(arg.Name)
-		ty := mod.typeString(codegen.OptionalType(arg), true, true /*acceptMapping*/)
-		fmt.Fprintf(w, "%s%s: %s = None,\n", ind, pname, ty)
-	}
-	fmt.Fprintf(w, "%sopts: Optional[pulumi.InvokeOptions] = None", indent)
-	if retTypeName != "" {
-		fmt.Fprintf(w, ") -> %s:\n", retTypeName)
-	} else {
-		fmt.Fprintf(w, "):\n")
-	}
-
-	// If this func has documentation, write it at the top of the docstring, otherwise use a generic comment.
-	docs := &bytes.Buffer{}
-	if fun.Comment != "" {
-		fmt.Fprintln(docs, codegen.FilterExamples(fun.Comment, "python"))
-	} else {
-		fmt.Fprintln(docs, "Use this data source to access information about an existing resource.")
-	}
-	if len(args) > 0 {
-		fmt.Fprintln(docs, "")
-		for _, arg := range args {
-			mod.genPropDocstring(docs, PyName(arg.Name), arg, true /*acceptMapping*/)
-		}
-	}
-	printComment(w, docs.String(), "    ")
-
-	if fun.DeprecationMessage != "" {
-		fmt.Fprintf(w, "    pulumi.log.warn(\"\"\"%s is deprecated: %s\"\"\")\n", name, fun.DeprecationMessage)
-	}
+	mod.genFunDef(w, name, retTypeName, args, false /* wrapInput */)
+	mod.genFunDocstring(w, fun)
+	mod.genFunDeprecationMessage(w, fun)
 
 	// Copy the function arguments into a dictionary.
 	fmt.Fprintf(w, "    __args__ = dict()\n")
@@ -1672,9 +1657,107 @@ func (mod *modContext) genFunction(fun *schema.Function) (string, error) {
 		}
 	}
 
+	mod.genFunctionOutputVersion(w, fun)
 	return w.String(), nil
 }
 
+func (mod *modContext) genFunDocstring(w io.Writer, fun *schema.Function) {
+	var args []*schema.Property
+	if fun.Inputs != nil {
+		args = fun.Inputs.Properties
+	}
+
+	// If this func has documentation, write it at the top of the docstring, otherwise use a generic comment.
+	docs := &bytes.Buffer{}
+	if fun.Comment != "" {
+		fmt.Fprintln(docs, codegen.FilterExamples(fun.Comment, "python"))
+	} else {
+		fmt.Fprintln(docs, "Use this data source to access information about an existing resource.")
+	}
+	if len(args) > 0 {
+		fmt.Fprintln(docs, "")
+		for _, arg := range args {
+			mod.genPropDocstring(docs, PyName(arg.Name), arg, true /*acceptMapping*/)
+		}
+	}
+	printComment(w, docs.String(), "    ")
+}
+
+func (mod *modContext) genFunDeprecationMessage(w io.Writer, fun *schema.Function) {
+	if fun.DeprecationMessage == "" {
+		return
+	}
+	name := PyName(tokenToName(fun.Token))
+	fmt.Fprintf(w, "    pulumi.log.warn(\"\"\"%s is deprecated: %s\"\"\")\n", name, fun.DeprecationMessage)
+}
+
+// Generates the function signature line `def fn(...):` without the body.
+func (mod *modContext) genFunDef(w io.Writer, name, retTypeName string, args []*schema.Property, wrapInput bool) {
+	def := fmt.Sprintf("def %s(", name)
+	var indent string
+	if len(args) > 0 {
+		indent = strings.Repeat(" ", len(def))
+	}
+	fmt.Fprintf(w, def)
+	for i, arg := range args {
+		var ind string
+		if i != 0 {
+			ind = indent
+		}
+		pname := PyName(arg.Name)
+
+		var argType schema.Type
+		if wrapInput {
+			argType = &schema.OptionalType{
+				ElementType: &schema.InputType{
+					ElementType: arg.Type,
+				},
+			}
+		} else {
+			argType = codegen.OptionalType(arg)
+		}
+
+		ty := mod.typeString(argType, true /*input*/, true /*acceptMapping*/)
+		fmt.Fprintf(w, "%s%s: %s = None,\n", ind, pname, ty)
+	}
+	fmt.Fprintf(w, "%sopts: Optional[pulumi.InvokeOptions] = None", indent)
+	if retTypeName != "" {
+		fmt.Fprintf(w, ") -> %s:\n", retTypeName)
+	} else {
+		fmt.Fprintf(w, "):\n")
+	}
+}
+
+// Generates `def ${fn}_output(..) version lifted to work on
+// `Input`-wrapped arguments and producing an `Output`-wrapped result.
+func (mod *modContext) genFunctionOutputVersion(w io.Writer, fun *schema.Function) {
+	if !fun.NeedsOutputVersion() {
+		return
+	}
+
+	var retTypeName string
+	if fun.Outputs != nil {
+		originalOutputTypeName, _ := awaitableTypeNames(fun.Outputs.Token)
+		retTypeName = fmt.Sprintf("pulumi.Output[%s]", originalOutputTypeName)
+	} else {
+		retTypeName = "pulumi.Output[void]"
+	}
+
+	originalName := PyName(tokenToName(fun.Token))
+	outputSuffixedName := fmt.Sprintf("%s_output", originalName)
+
+	var args []*schema.Property
+	if fun.Inputs != nil {
+		args = fun.Inputs.Properties
+	}
+
+	fmt.Fprintf(w, "\n\n@_utilities.lift_output_func(%s)\n", originalName)
+	mod.genFunDef(w, outputSuffixedName, retTypeName, args, true /*wrapInput*/)
+	mod.genFunDocstring(w, fun)
+	mod.genFunDeprecationMessage(w, fun)
+	fmt.Fprintf(w, "    ...\n")
+}
+
 func (mod *modContext) genEnums(w io.Writer, enums []*schema.EnumType) error {
 	// Header
 	mod.genHeader(w, false /*needsSDK*/, nil)
@@ -1729,7 +1812,7 @@ func (mod *modContext) genEnum(w io.Writer, enum *schema.EnumType) error {
 				fmt.Fprintf(w, "%v\n", e.Value)
 			}
 			if e.Comment != "" {
-				fmt.Fprintf(w, "%s\"\"\"%s\"\"\"\n", indent, e.Comment)
+				printComment(w, e.Comment, indent)
 			}
 		}
 	default:
@@ -2684,11 +2767,13 @@ func GeneratePackage(tool string, pkg *schema.Package, extraFiles map[string][]b
 }
 
 const utilitiesFile = `
+import importlib.util
+import inspect
 import json
 import os
-import sys
-import importlib.util
 import pkg_resources
+import sys
+import typing
 
 import pulumi
 import pulumi.runtime
@@ -2890,4 +2975,26 @@ def register(resource_modules, resource_packages):
             mod_info['pkg'],
             mod_info['mod'],
             Module(mod_info))
+
+
+_F = typing.TypeVar('_F', bound=typing.Callable[..., typing.Any])
+
+
+def lift_output_func(func: typing.Any) -> typing.Callable[[_F], _F]:
+    """Decorator internally used on {fn}_output lifted function versions
+    to implement them automatically from the un-lifted function."""
+
+    func_sig = inspect.signature(func)
+
+    def lifted_func(*args, opts=None, **kwargs):
+        bound_args = func_sig.bind(*args, **kwargs)
+
+        return pulumi.Output.from_input({
+            'args': bound_args.args,
+            'kwargs': bound_args.kwargs
+        }).apply(lambda resolved_args: func(*resolved_args['args'],
+                                            opts=opts,
+                                            **resolved_args['kwargs']))
+
+    return (lambda _: lifted_func)
 `
