@@ -642,10 +642,7 @@ namespace Pulumi.Automation
         /// supported for local backends.
         /// </summary>
         public async Task CancelAsync(CancellationToken cancellationToken = default)
-        {
-            await this.Workspace.RunCommandAsync(new[] { "cancel", "--stack", this.Name, "--yes" }, cancellationToken)
-                .ConfigureAwait(false);
-        }
+            => await this.Workspace.RunCommandAsync(new[] { "cancel", "--stack", this.Name, "--yes" }, cancellationToken).ConfigureAwait(false);
 
         private async Task<CommandResult> RunCommandAsync(
             IList<string> args,
@@ -655,7 +652,7 @@ namespace Pulumi.Automation
             CancellationToken cancellationToken)
         {
             args = args.Concat(new[] { "--stack", this.Name }).ToList();
-            return await this.Workspace.RunStackCommandAsync(this.Name, args, onStandardOutput, onStandardError, onEngineEvent, cancellationToken);
+            return await this.Workspace.RunStackCommandAsync(this.Name, args, onStandardOutput, onStandardError, onEngineEvent, cancellationToken).ConfigureAwait(false);
         }
 
         public void Dispose()
