@@ -407,16 +407,6 @@ func TestDynamicPython(t *testing.T) {
 	})
 }
 
-func TestDynamicPythonNonMain(t *testing.T) {
-	integration.ProgramTest(t, &integration.ProgramTestOptions{
-		Dir: filepath.Join("dynamic", "python-non-main"),
-		Dependencies: []string{
-			filepath.Join("..", "..", "sdk", "python", "env", "src"),
-		},
-	})
-}
-
-
 // Tests custom resource type name of dynamic provider in Python.
 func TestCustomResourceTypeNameDynamicPython(t *testing.T) {
 	integration.ProgramTest(t, &integration.ProgramTestOptions{
@@ -433,7 +423,6 @@ func TestCustomResourceTypeNameDynamicPython(t *testing.T) {
 	})
 }
 
-
 func TestPartialValuesPython(t *testing.T) {
 	if runtime.GOOS == WindowsOS {
 		t.Skip("Temporarily skipping test on Windows - pulumi/pulumi#3811")
@@ -1124,3 +1113,22 @@ func TestBrokenDynamicProvider(t *testing.T) {
 		ExpectFailure: true,
 	})
 }
+
+// Test that the about command works as expected. Because about parses the
+// results of each runtime independently, we have an integration test in each
+// language.
+func TestAboutPython(t *testing.T) {
+	dir := filepath.Join("about", "python")
+
+	e := ptesting.NewEnvironment(t)
+	defer func() {
+		if !t.Failed() {
+			e.DeleteEnvironmentFallible()
+		}
+	}()
+	e.ImportDirectory(dir)
+
+	stdout, _ := e.RunCommand("pulumi", "about", "--json")
+	// Assert we parsed the dependencies
+	assert.Contains(t, stdout, "pulumi-kubernetes")
+}
